@@ -25,6 +25,10 @@ _histograms = (
 )
 
 
+def _hist_or_val(other):
+    return other._hist if hasattr(other, "_hist") else other
+
+
 def _arg_shortcut(item):
     msg = "Developer shortcut: will be removed in a future version"
     if isinstance(item, tuple) and len(item) == 3:
@@ -129,7 +133,11 @@ class BaseHistogram(object):
         return _to_view(self._hist.view(False))
 
     def __add__(self, other):
-        return self.__class__(self._hist + other._hist)
+        return self.__class__(self._hist.__add__(other._hist))
+
+    def __iadd__(self, other):
+        self._hist.__iadd__(other._hist)
+        return self
 
     def __eq__(self, other):
         return self._hist == other._hist
@@ -139,28 +147,28 @@ class BaseHistogram(object):
 
     # If these fail, the underlying object throws the correct error
     def __mul__(self, other):
-        if isinstance(other, BaseHistogram):
-            return self.__class__(self._hist * other._hist)
-        else:
-            return self.__class__(self._hist * other)
+        return self.__class__(self._hist.__mul__(other))
 
     def __rmul__(self, other):
         return self * other
 
     def __imul__(self, other):
-        return self.__class__(self._hist.__imul__(other))
+        self._hist.__imul__(_hist_or_val(other))
+        return self
 
     def __truediv__(self, other):
-        return self.__class__(self._hist.__truediv__(other._hist))
+        return self.__class__(self._hist.__truediv__(_hist_or_val(other)))
 
     def __div__(self, other):
-        return self.__class__(self._hist.__div__(other._hist))
+        return self.__class__(self._hist.__div__(_hist_or_val(other)))
 
     def __itruediv__(self, other):
-        return self.__class__(self._hist.__itruediv__(other._hist))
+        self._hist.__itruediv__(_hist_or_val(other))
+        return self
 
     def __idiv__(self, other):
-        return self.__class__(self._hist.__idiv__(other._hist))
+        self._hist.__idiv__(_hist_or_val(other))
+        return self
 
     def __copy__(self):
         other = self.__class__.__new__(self.__class__)
@@ -477,12 +485,18 @@ class Histogram(BaseHistogram):
         zeroes_start = []
         zeroes_stop = []
 
+        # We could use python's sum here, but for now, a private sum is used
+        class ext_sum:
+            projection = True
+
         # Compute needed slices and projections
         for i, ind in iterator:
-            if not isinstance(ind, slice):
+            if hasattr(ind, "__index__"):
+                ind = slice(ind.__index__(), ind.__index__() + 1, ext_sum())
+
+            elif not isinstance(ind, slice):
                 raise IndexError(
-                    "Invalid arguments as an index, use all integers "
-                    "or all slices, and do not mix"
+                    "Must be a slice, an integer, or follow the locator protocol."
                 )
             if ind != slice(None):
                 merge = 1
@@ -573,11 +587,15 @@ class Histogram(BaseHistogram):
         # Numpy does not broadcast partial slices, but we would need
         # to allow it (because we do allow broadcasting up dimensions)
         # Instead, we simply require matching dimensions.
-        if value.ndim > 0 and value.ndim != len(indexes):
+        if value.ndim > 0 and value.ndim != sum(isinstance(i, slice) for i in indexes):
             raise ValueError(
-                "Setting a histogram with an array must have a matching number of dimensions"
+                "Setting a {0}D histogram with a {1}D array must have a matching number of dimensions".format(
+                    len(indexes), value.ndim
+                )
             )
 
+        # Here, value_n does not increment with n if this is not a slice
+        value_n = 0
         for n, request in iterator:
             has_underflow = self.axes[n].options.underflow
             has_overflow = self.axes[n].options.overflow
@@ -592,16 +610,26 @@ class Histogram(BaseHistogram):
                 stop = len(self.axes[n]) if request.stop is None else request.stop
                 request_len = stop - start
 
-                # If there are not enough dimensions, then treat it like broadcasting
-                if value.ndim == 0 or value.shape[n] == 1:
+                # If set to a scalar, then treat it like broadcasting without flow bins
+                if value.ndim == 0:
                     start = 0 + has_overflow
                     stop = len(self.axes[n]) + has_underflow
-                elif request_len == value.shape[n]:
+
+                # Normal setting
+                elif request_len == value.shape[value_n]:
                     start += has_underflow
                     stop += has_underflow
-                elif request_len + use_underflow + use_overflow == value.shape[n]:
+
+                # Expanded setting
+                elif request_len + use_underflow + use_overflow == value.shape[value_n]:
                     start += has_underflow and not use_underflow
                     stop += has_underflow + (has_overflow and use_overflow)
+
+                # Single element broadcasting
+                elif value.shape[value_n] == 1:
+                    start += has_underflow
+                    stop += has_underflow
+
                 else:
                     msg = "Mismatched shapes in dimension {0}".format(n)
                     msg += ", {0} != {1}".format(value.shape[n], request_len)
@@ -611,6 +639,7 @@ class Histogram(BaseHistogram):
                         )
                     raise ValueError(msg)
                 indexes[n] = slice(start, stop, request.step)
+                value_n += 1
             else:
                 indexes[n] = request + has_underflow
 
